package me.hammerle.supersnuvi.util;

import me.hammerle.supersnuvi.tiles.Tile;

public class CollisionBox 
{
    public final static CollisionBox NULL_BOX = new CollisionBox(0.0f, 0.0f, 0.0f, 0.0f);
    public final static CollisionBox DEFAULT_TILE_BOX = createScaledTileBox(0.0f, 0.0f, 1.0f, 1.0f);
    
    private float minX;
    private float maxX;
    private float minY;
    private float maxY;
    
    private float cMinX;
    private float cMaxX;
    private float cMinY;
    private float cMaxY;
    
    public CollisionBox(float x1, float y1, float x2, float y2)
    {
        minX = Math.min(x1, x2);
        maxX = Math.max(x1, x2);
        minY = Math.min(y1, y2);
        maxY = Math.max(y1, y2);
        cMinX = minX;
        cMaxX = maxX;
        cMinY = minY;
        cMaxY = maxY;
    }
    
    public static CollisionBox createScaledTileBox(float x1, float y1, float x2, float y2)
    {
        return new CollisionBox(x1 * Tile.SIZE, y1 * Tile.SIZE, x2 * Tile.SIZE, y2 * Tile.SIZE);
    }
    
    public CollisionBox copy()
    {
        return new CollisionBox(cMinX, cMinY, cMaxX, cMaxY);
    }
    
    public void save()
    {
        minX = cMinX;
        maxX = cMaxX;
        minY = cMinY;
        maxY = cMaxY;
    }
    
    public CollisionBox reset()
    {
        cMinX = minX;
        cMaxX = maxX;
        cMinY = minY;
        cMaxY = maxY;
        return this;
    }

    public float getWidth() 
    {
        return maxX - minX;
    }
    
    public float getHeight() 
    {
        return maxY - minY;
    }
    
    public float getMinX() 
    {
        return cMinX;
    }

    public float getMaxX() 
    {
        return cMaxX;
    }

    public float getMinY() 
    {
        return cMinY;
    }

    public float getMaxY() 
    {
        return cMaxY;
    }
   
    public CollisionBox expand(float x, float y)
    {
        if(x < 0.0f)
        {
            cMinX += x;
        }
        else
        {
            cMaxX += x;
        }
        if(y < 0.0f)
        {
            cMinY += y;
        }
        else
        {
            cMaxY += y;
        }
        return this;
    }

    public CollisionBox grow(float x, float y)
    {
        x = Math.abs(x);
        y = Math.abs(y);
        cMinX -= x;
        cMaxX += x;
        cMinY -= y;
        cMaxY += y;
        return this;
    }

    public CollisionBox grow(float value)
    {
        return this.grow(value, value);
    }
    
    public CollisionBox offset(float x, float y)
    {
        cMinX += x;
        cMaxX += x;
        cMinY += y;
        cMaxY += y;
        return this;
    }
    
    public CollisionBox offsetX(float x)
    {
        cMinX += x;
        cMaxX += x;
        return this;
    }
    
    public CollisionBox offsetY(float y)
    {
        cMinY += y;
        cMaxY += y;
        return this;
    }
    
    public boolean isColliding(float minX, float minY, float maxX, float maxY)
    {
        return cMaxX > minX && maxX > cMinX && cMaxY > minY && maxY > cMinY;
    }
    
    public boolean isColliding(CollisionBox cb)
    {
        return isColliding(cb.cMinX, cb.cMinY, cb.cMaxX, cb.cMaxY);
    }

    public CollisionBox intersect(CollisionBox cb)
    {
        return new CollisionBox(Math.max(cMinX, cb.cMinX), Math.max(cMinY, cb.cMinY), 
                Math.min(cMaxX, cb.cMaxX), Math.min(cMaxY, cb.cMaxY));
    }

    @Override
    public String toString() 
    {
        StringBuilder sb = new StringBuilder("box[x1 = ");
        sb.append(cMinX);
        sb.append(", y1 = ");
        sb.append(cMinY);
        sb.append(", x2 = ");
        sb.append(cMaxX);
        sb.append(", y2 = ");
        sb.append(cMaxY);
        sb.append(']');
        return sb.toString();
    } 
}
